package PureMessage::Policy::Virus::Sophos;

use strict;

our $VERSION = '6.301';

use base 'PureMessage::Policy::Virus';
use Sophos::SAVI::Engine;
use PureMessage::Logger;

sub get_engine {
    my $my_config = shift;

    my ($datadir, %opts);
    while (my ($k, $v) = each %$my_config) {
	next if $k eq 'enabled' or $k eq 'daemon';
	if ($k eq 'datadir') {
	    $datadir = $v;
	    next;
	}
	$opts{$k} = $v if defined $v;
    }
    $opts{logger} = PureMessage::Logger->new(ident=>"Sophos-Engine");
    my %RANK = (
	GrpSuper		=> 2,
	GrpArchiveUnpack	=> 1,
	GrpSelfExtract		=> 1,
	GrpExecutable		=> 1,
	GrpInternet		=> 1,
	GrpMSOffice		=> 1,
	GrpMisc			=> 1,
	GrpDisinfect		=> 1,
	GrpClean		=> 1,
    );
    my @opts = map { $_ => $opts{$_} }
	       sort { ($RANK{$b}||0) <=> ($RANK{$a}||0) || ($a cmp $b) }
	       keys %opts;

    # Create the engine here, and let the closures pick it up.
    Sophos::SAVI::Engine->new($datadir, @opts);
}

sub scan_context{
    my ($engine, $msg,$version) = @_;
    my %property ;
    my ($part,$val,$scan_context);

    if ($msg->part_id eq  '1' ) {   # for whole message including headers
        $scan_context = $engine->set_scan_context('Genes/Extn/ProdVer',"PMX:Email:$version");
        $scan_context = $engine->set_scan_context('Email/MIME/Content-Type','message/rfc822');
        $property{'Genes/Extn/ProdVer'} = '1';
        $property{'Email/MIME/Content-Type'} = '1';
        return %property;
     } else {
        $part=$msg;
     }
     $scan_context = $engine->set_scan_context('Genes/Extn/ProdVer',"PMX:Email:$version");
     $property{'Genes/Extn/ProdVer'} = '1';

     $val= $part->header_decoded('Content-Disposition');
     if( defined $val ) {
         $scan_context = $engine->set_scan_context('Email/MIME/Content-Disposition',$val);
         $property{'Email/MIME/Content-Disposition'} = '1';
     }

     $val= $part->header_decoded('Content-Type');
     if( defined $val ) {
        $scan_context = $engine->set_scan_context('Email/MIME/Content-Type',$val);
        $property{'Email/MIME/Content-Type'} = '1';
     }
     return %property ;

}

sub delete_property{
   my ($engine,%property) = @_;
   my $status ;
   for my $property (keys %property) {
        $status = $engine->delete_scan_property($property);
   }
   return $status;
}


sub impl {
    my ($pkg, $my_config) = @_;
    my $engine = get_engine($my_config);
    require PureMessage::Version;
    my $version = PureMessage::Version::Version();

    [
	sub {
	    my ($self, $msg, $filename) = @_;
	    my @viruses;
	     my (%property, $status);

            # set scan properties on scan context
            %property = scan_context($engine, $msg, $version);

            my $report = $engine->scan_file($filename);

            # Report hash is list of viruses found for each (part) filename -
            # we only want the complete list
            @viruses = map { @$_ } values %$report;

            $status = delete_property($engine, %property);
            return @viruses;

	},
	# Clean: must return one of qw(novirus cleanfailed cleaned deleted)
        # We can't try to clean the full message, because we can't reconstruct
        # the headers after we've cleaned the body
	sub {
            my ($self, $msg, $filename) = @_;
	    my %property;
	    %property = scan_context($engine, $msg, $version);
            my $report = $engine->clean_file($filename);
	    delete_property($engine, %property);
            my $status = $report->{$filename};
	},
    ]
}

sub daemon {
    my ($pkg, $my_config) = @_;
    my $engine = get_engine($my_config);

    require PureMessage::Version;
    my $version = PureMessage::Version::Version();

    [
	sub {
	    my $self = shift;
            my %property = @_;
            my $filename = delete $property{file};
            my (@viruses, $scan_context, $status);
            $scan_context = $engine->set_scan_context('Genes/Extn/ProdVer',"PMX:Email:$version");
            for my $key ( keys %property) {
                # set scan context property and value
                $scan_context = $engine->set_scan_context($key,$property{$key});
            }
            my $report = $engine->scan_file($filename);

            # Report hash is list of viruses found for each (part) filename -
            # we only want the complete list
            @viruses = map { @$_ } values %$report;

            $property{'Genes/Extn/ProdVer'} = '1';
            # Delete the property set on savi object
            $status = delete_property($engine,%property);
            return @viruses;

	},
	# Clean: must return one of qw(novirus cleanfailed cleaned deleted)
	sub {
	    my ($self, $filename) = @_;
	    my $scan_context = $engine->set_scan_context('Genes/Extn/ProdVer',"PMX:Email:$version");	#change
	    my $report = $engine->clean_file($filename);
	    $engine->delete_scan_property('Genes/Extn/ProdVer');					#change
	    my $status = $report->{$filename};
	    return $status;
	},
    ]
}

1;
